@@ -1,12 +1,25 @@
+from typing import Any, Dict, Optional, Union
+
 import arviz as az
+import matplotlib.pyplot as plt
+import numpy as np
+import pandas as pd
+import xarray as xr
 
 
 def plot_xY(
-    x, Y, ax, plot_hdi_kwargs=dict(), hdi_prob: float = 0.94, include_label: bool = True
+    x: Union[pd.DatetimeIndex, np.array],
+    Y: xr.DataArray,
+    ax: plt.Axes,
+    plot_hdi_kwargs: Optional[Dict[str, Any]] = None,
+    hdi_prob: float = 0.94,
+    include_label: bool = True,
 ) -> None:
     """Utility function to plot HDI intervals."""
 
-    Y = Y.stack(samples=["chain", "draw"]).T
+    if plot_hdi_kwargs is None:
+        plot_hdi_kwargs = {}
+
     az.plot_hdi(
         x,
         Y,
@@ -21,7 +34,7 @@ def plot_xY(
     )
     ax.plot(
         x,
-        Y.mean(dim="samples"),
+        Y.mean(dim=["chain", "draw"]),
         color="k",
         label="Posterior mean" if include_label else None,
     )
